@@ -3,6 +3,7 @@ from pathlib import Path
 
 import alembic
 import alembic.config
+from humanize import naturaldelta
 from sqlalchemy import (
     BigInteger,
     Column,
@@ -20,6 +21,8 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import Session, relationship, sessionmaker
 from sqlalchemy.sql.expression import label
 
+from spellbot.constants import AVG_QUEUE_TIME_WINDOW_MIN
+
 PACKAGE_ROOT = Path(__file__).resolve().parent
 ASSETS_DIR = PACKAGE_ROOT / "assets"
 ALEMBIC_INI = ASSETS_DIR / "alembic.ini"
@@ -96,7 +99,7 @@ class User(Base):
 
     @property
     def waiting(self):
-        return self.game is not None
+        return self.queued_at is not None
 
     def enqueue(self, *, server, channel_xid, include, size, power, tags):
         session = Session.object_session(self)
@@ -112,6 +115,10 @@ class User(Base):
         valid_game_ids = []
         for row in considerations:
             game = session.query(Game).get(row.game_id)
+            if not game:
+                continue
+            if game.url is not None:
+                continue
             if set(tag.id for tag in game.tags) != required_tag_ids:
                 continue
             if len(game.users) >= size - len(include):
@@ -138,54 +145,95 @@ class User(Base):
             .order_by(Game.created_at)
             .first()
         )
+        now = datetime.utcnow()
+        expires_at = now + timedelta(minutes=server.expire)
         if existing_game:
             self.game = existing_game
+            self.game.updated_at = now
+            self.game.expires_at = expires_at
         else:
             self.game = Game(
-                size=size,
-                guild_xid=guild_xid,
                 channel_xid=channel_xid if server.scope == "channel" else None,
+                created_at=now,
+                expires_at=expires_at,
+                guild_xid=guild_xid,
                 power=power,
+                size=size,
                 tags=tags,
+                updated_at=now,
             )
             session.add(self.game)
-        queued_at = datetime.utcnow()
-        self.queued_at = queued_at
+        self.queued_at = now
         for user in include:
             user.game = self.game
-            user.queued_at = queued_at
+            user.queued_at = now
 
     def dequeue(self):
         session = Session.object_session(self)
-        if len(self.game.users) == 1:
+        if self.game and len(self.game.users) == 1:
             session.delete(self.game)
         self.game = None
+        self.queued_at = None
 
 
 class Game(Base):
     __tablename__ = "games"
     id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
     created_at = Column(DateTime, nullable=False, default=datetime.utcnow)
+    updated_at = Column(DateTime, default=datetime.utcnow)
+    expires_at = Column(DateTime)
     size = Column(Integer, nullable=False)
     guild_xid = Column(
         BigInteger, ForeignKey("servers.guild_xid", ondelete="CASCADE"), nullable=False
     )
     channel_xid = Column(BigInteger)
     power = Column(Integer)
+    url = Column(String(255))
     users = relationship("User", back_populates="game")
     tags = relationship("Tag", secondary=games_tags, back_populates="games")
     server = relationship("Server", back_populates="games")
 
     @classmethod
     def expired(cls, session):
-        now = datetime.utcnow()
-        if "sqlite" in session.bind.driver:
-            comp = text(f"datetime('{now}', '-' || servers.expire || ' minutes')")
+        return (
+            session.query(Game)
+            .filter(and_(datetime.utcnow() >= Game.expires_at, Game.url == None))
+            .all()
+        )
+
+    def to_str(self):
+        session = Session.object_session(self)
+        rvalue = ""
+        if self.url:
+            rvalue += "**Your SpellTable game is ready!**\n"
+            rvalue += f"{self.url}\n"
         else:
-            comp = text(
-                f"CAST('{now}' AS TIMESTAMP) - (servers.expire * interval '1 minute')"
+            rvalue += (
+                "**You have been entered in a play queue "
+                f"for a {self.size} player game.**"
             )
-        return session.query(Game).join(Server).filter(Game.created_at < comp).all()
+            average = WaitTime.average(
+                session,
+                guild_xid=self.server.guild_xid,
+                channel_xid=self.channel_xid,
+                scope=self.server.scope,
+                window_min=AVG_QUEUE_TIME_WINDOW_MIN,
+            )
+            if average:
+                delta = naturaldelta(timedelta(seconds=average))
+                rvalue += f" _The average wait time is {delta}._\n"
+            else:
+                rvalue += "\n"
+        players = ", ".join(sorted([f"<@{user.xid}>" for user in self.users]))
+        rvalue += f"Players: {players}\n"
+        if self.channel_xid:
+            rvalue += f"Channel: <#{self.channel_xid}>\n"
+        if not (len(self.tags) == 1 and self.tags[0].name == "default"):
+            tag_names = ", ".join(sorted([tag.name for tag in self.tags]))
+            rvalue += f"Tags: {tag_names}\n"
+        if self.power:
+            rvalue += f"Power Level: {self.power}\n"
+        return rvalue.strip()
 
 
 class Tag(Base):
